<?php
/**
 * @package			No Boss Extensions
 * @subpackage  	No Boss Library
 * @author			No Boss Technology <contact@nobosstechnology.com>
 * @copyright		Copyright (C) 2024 No Boss Technology. All rights reserved.
 * @license			GNU Lesser General Public License version 3 or later; see <https://www.gnu.org/licenses/lgpl-3.0.en.html>
 */

use Joomla\Filesystem\File;
use Joomla\CMS\Factory;

// phpcs:disable PSR1.Files.SideEffects
\defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

/**
 * Script de instalação da library utilizada por todas extensoes da No Boss
 * - Esse script normalmente é executado após o script de instalacao do pacote
 * 
 * - Formatos para retornar mensagens de erro / warning:
 *     // Mensagem de erro (fundo vermelho), sem interromper a instalacao'
 *     Factory::getApplication()->enqueueMessage(Text::_("Aqui mensagem"), 'Error');
 *           
 *     // Mensagem de warning (fundo amarelo), sem interromper a instalacao
 *     Factory::getApplication()->enqueueMessage(Text::_("Aqui mensagem"), 'Warning');
 *           
 *     // Mensagem de warning (fundo amarelo), interrompendo a instalacao (Joomla tb exibe mensagem de erro da instalacao)
 *     throw new RuntimeException(Text::_("Aqui mensagem"));
 *     return false;
 * 
 * 
 * - Exemplos de usos do $parent que é recebido como parametro nas funções:
 *     // Obter nova versao recem instalada
 *     $parent->get('manifest')->version;
 * 
 *     // Obter alias da extensao (Ex: 'mod_nobosscalendar)
 *     $parent->manifest->name;
 *       
 *     // Redirecionar usuario apos a instalacao (interessante usar no metodo install na instalacao de componentes nossos para redirecionar o usuario para a pagina principal do componente apos a instalacao)
 *     $parent->getParent()->setRedirectURL('index.php?option=com_helloworld');*
 *
 * 
 * - Sobre uso de constantes de tradução
 *    - Se você deseja que as KEYs desses idiomas sejam usadas na primeira instalação do componente, o arquivo de idioma .sys.ini deve ser armazenado na pasta do componente (admin/language/en-GB/en-GB.com_helloworld.sys.ini)
 *    - Quando ja temos a library instalada, podemos forcar o carregamento do idioma da library e usar as constantes dela
 */

// TODO: tratamento necessario pq usuario pode estar com versao antiga (sem /src) da library e nestes casos geraria erro
// Library encontrada no novo formato j4 / j5
if(is_dir(JPATH_LIBRARIES.'/noboss/src')){
    require_once JPATH_LIBRARIES."/noboss/src/Util/NbInstallScriptUtil.php";
    require_once JPATH_LIBRARIES."/noboss/src/Util/NbCurlUtil.php"; // Esse eh necessario pq eh usado dentro da classe NbInstallScriptUtil
    require_once JPATH_LIBRARIES."/noboss/src/Form/Field/Nblicense/NblicenseModel.php";
}
// Library ainda esta no formato antigo sem pasta /src
else if(is_dir(JPATH_LIBRARIES.'/noboss/util')){
    require_once JPATH_LIBRARIES."/noboss/util/installscript.php";
    require_once JPATH_LIBRARIES."/noboss/forms/fields/nobosslicense/nobosslicensemodel.php";
}

class mod_NobossheaderInstallerScript{

    /**
     * Evento executado antes de qualquer outro no processo de instalacao / update
     *  - Esse é o momento que a instalacao / update pode ser cancelado
     *  - Antes dessa funcao, apenas o preflight do script de instalacao do pacote é executado
     *
     * @param   string     $type        Tipo de intalações (install, update, discover_install)
     * @param   JInstaller $parent      Classe que chama esse metodo (pode acessar funcoes dela)
     *
     * @return  boolean  true caso deva ser instalada, false para cancelar a instalação
     */
    function preflight($type, $parent){
        if($type == 'uninstall'){
            return;
        }

        // Library nao foi instalado
        if(!is_dir(JPATH_LIBRARIES.'/noboss')) {
            throw new RuntimeException('In order to install the extension, first install No Boss Library using the url https://www.nobossextensions.com/en/installation/nobosslibrary', 500);
        }
        // No Boss Ajax nao foi instalado
        else if(!is_dir(JPATH_ROOT.'/components/com_nobossajax/')){
            throw new RuntimeException('In order to install the extension, first install No Boss Ajax using the url https://www.nobossextensions.com/en/installation/nobossajax', 500);
        }

        // Arquivo do zip de instalacao / update com o token da licenca
        $extraInfoPath = __DIR__ . '/extra-info.json';

        // Confirma se arquivo existe no zip de instalacao
        if(is_file($extraInfoPath)){
            // Lê o conteudo do json que contem o token
            $this->extraInfo = json_decode(file_get_contents($extraInfoPath), true);
            // Monta um array apenas com as informações que serão salvas na coluna extra_query do banco
            $extraQueryArray = array('token' => $this->extraInfo['token']);

            // Library encontrada no novo formato j4 / j5
            if(is_dir(JPATH_LIBRARIES.'/noboss/src')){
                // Salva o token na coluna extra_query do banco
                Noboss\Library\Util\NbInstallScriptUtil::updateExtraQuery($parent, http_build_query($extraQueryArray));
            }
            // Library ainda esta no formato antigo sem pasta /src
            else if(is_dir(JPATH_LIBRARIES.'/noboss/util')){
                // Salva o token na coluna extra_query do banco
                NoBossUtilInstallscript::updateExtraQuery($parent, http_build_query($extraQueryArray));
            }
        }
    }

    /**
     * Metodo executado após o término da instalação / update
     * 
     * @param   string     $type        Tipo de intalações (install, update, discover_install)
     * @param   JInstaller $parent      Classe que chama esse metodo (pode acessar funcoes dela)
     */
    public function postflight($type, $parent) {
        if($type == 'uninstall'){
            return;
        }

        $extensionName = $parent->manifest->name;

        // Library encontrada no novo formato j4 / j5
        if(is_dir(JPATH_LIBRARIES.'/noboss/src')){
            $tokenArray = Noboss\Library\Form\Field\Nblicense\NblicenseModel::getLicenseTokenAndPlan($extensionName);
            $token = array_key_exists("token", $tokenArray) ? $tokenArray['token'] : '';

            // Salva a url autorizada
            Noboss\Library\Util\NbInstallScriptUtil::saveAuthorizedUrl($token);
        }
        // Library ainda esta no formato antigo sem pasta /src
        else if(is_dir(JPATH_LIBRARIES.'/noboss/util')){
            $tokenArray = NobossModelNobosslicense::getLicenseTokenAndPlan($extensionName);
            $token = array_key_exists("token", $tokenArray) ? $tokenArray['token'] : '';

            // Salva a url autorizada
            NoBossUtilInstallscript::saveAuthorizedUrl($token);
        }

        // Eh Update
        if($type == 'update'){
            // Library encontrada no novo formato j4 / j5
            if(is_dir(JPATH_LIBRARIES.'/noboss/src')){
                // Verifica se a library esta na lista de atualizacoes pendentes
                $libUpdateId = Noboss\Library\Util\NbInstallScriptUtil::getLibraryUpdateId($parent);
            }
            // Library ainda esta no formato antigo sem pasta /src
            else if(is_dir(JPATH_LIBRARIES.'/noboss/util')){
                // Verifica se a library esta na lista de atualizacoes pendentes
                $libUpdateId = NoBossUtilInstallscript::getLibraryUpdateId($parent);
            }

            // Library precisa ser atualizada
            if(!empty($libUpdateId)){
                // Diretorios da library estao sem permissao de escrita
                if(!Joomla\Filesystem\Path::canChmod(JPATH_SITE.'/libraries/noboss/') || !Joomla\Filesystem\Path::canChmod(JPATH_SITE.'/layouts/noboss/')){
                    // Seta mensagem a ser exibida orientando sobre o problema
                    $urlLibrary = 'https://www.nobossextensions.com/installation/nobosslibrary';
                    $urlDocPermission = 'https://docs.nobosstechnology.com/extensoes-joomla/ajustando-permissoes-de-diretorios-e-arquivos-no-joomla';
                    $message = "It was not possible to update No Boss Library (library used by the extension) due to lack of permissions in the '/libraries/noboss/' and '/layouts/noboss/' directories. <br> <br> Correct the permissions of the directories and after updating the library on the page of <a href='index.php?option=com_installer&view=update' target='_blank'>extension updates</a>. It is also possible to update by installing the library again from the url <a href='{$urlLibrary}' target='_blank'>{$urlLibrary}</a>. <br> <br> For information on how to fix permissions, access the <a href='{$urlDocPermission}' target='_blank'>documentation on setting Joomla file and directory permissions</a>.";

                    // Exibe mensagem como warning (fundo amarelo)
                    Factory::getApplication()->enqueueMessage($message, 'Warning');
                }
                // Realiza atualizacao da library
                else{
                    // JModelLegacy::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_installer/models');
                    // $updateModel = JModelLegacy::getInstance('update', 'InstallerModel');
                    $updateModel = Factory::getApplication()->bootComponent('com_installer')->getMVCFactory(Factory::getApplication())->createModel('Update', 'Administrator');

                    // Realiza o update da library usando o id dela
                    $updateModel->update(array($libUpdateId));
                }
            }
        }
    }
    
    /**
     * Metodo executado apos a instalacao
     *  - Aqui podemos exibir textos fora de notices / warnings utilizanco 'echo' ou html direto
     * 
     * @param   JInstaller $parent      Classe que chama esse metodo (pode acessar funcoes dela)
     */
	function install($parent) {
	}
    
    /**
     * Metodo executado apos a atualizacao
     * - OBS: aqui nao pega warning ou 'echo'
     * - Utilizar para colocar atualizacoes especificas de versoes (Ex: migrar dado no banco de estrutura antiga para nova)
     * - Para qual versao foi instalada (usar em alguma condicao), utilize $parent->get('manifest')->version
     * 
     * @param   JInstaller $parent      Classe que chama esse metodo (pode acessar funcoes dela)
     */
	function update($parent) {
        // TODO: Funcao que migra Joomla para v5
        //self::updateJoomla5();
    }

    /**
     * Método  de desistalação  da  extensão
     *
     * @param   JInstallerAdapterPackage   $parent é a classe chamando esse método
     *
     * @return void
     */
    function uninstall($parent){
        // Diretorio da 'No boss library' nao foi encontrado: nao sera possivel fazer verificacao de extensoes filhas
        if(!is_dir(JPATH_LIBRARIES.'/noboss/')) {
            return;
        }

        // Nome da extensao (ex: mod_nobosscalendar)
        $extensionName = strtolower($parent->manifest->name);
        
        // Library encontrada no novo formato j4 / j5
        if(is_dir(JPATH_LIBRARIES.'/noboss/src')){
            // Obtem token da extensao na base de dados
            $tokenArray = Noboss\Library\Form\Field\Nblicense\NblicenseModel::getLicenseTokenAndPlan($extensionName);

            $token = array_key_exists("token", $tokenArray) ? $tokenArray['token'] : '';

            if(!empty($token)){
                // Remove a url do site do usuario na plataforma da No Boss Extensions
                Noboss\Library\Util\NbInstallScriptUtil::removeUrlSite($token);
            }
        }
        // Library ainda esta no formato antigo sem pasta /src
        else if(is_dir(JPATH_LIBRARIES.'/noboss/util')){
            // Obtem token da extensao na base de dados
            $tokenArray = NobossModelNobosslicense::getLicenseTokenAndPlan($extensionName);

            $token = array_key_exists("token", $tokenArray) ? $tokenArray['token'] : '';
            
            if(!empty($token)){
                // Remove a url do site do usuario na plataforma da No Boss Extensions
                NoBossUtilInstallscript::removeUrlSite($token);
            }
        }
    }

    /*
    //     TODO: funcao temporaria criada para retirar prefixo 'model1' dos xml's por termos dividido em 3 models
    //             * Executado a partir da versao 2.1.2 criada em Junho de 2020
    //             * Remover apos Abril de 2021
    // */
    // function updateVersion2_1_2(){
    //     jimport('joomla.filesystem.file');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/subforms/model1_accessibility.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/subforms/model1_action_links.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/subforms/model1_additional_content.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/subforms/model1_contact_info.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/subforms/model1_items_manual.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/subforms/model1_language.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/subforms/model1_logo.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/subforms/model1_search.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/subforms/model1_slim_bar_logo.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/subforms/model1_social_media.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/subforms/model1_module_position.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/modals/model1_main_bar_customization.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/modals/model1_mobile_customization.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/modals/model1_secondary_bar_customization.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/modals/model1_slim_bar_customization.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/modals/model1_items_customization.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/modals/model1_menu_customization.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/tmpl/theme/model1_mobile.php');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/tmpl/theme/model1.php');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/tmpl/style/model1.php');

    //     $db = JFactory::getDBO();
    //     $query = $db->getQuery(true);

    //     $query
    //     ->select("id, params")
    //     ->from('#__modules')
    //     ->where("module = 'mod_nobossheader'");

    //     // Seta query
	// 	$db->setQuery($query);
		
	// 	//echo str_replace('#__', 'ext_', $query); exit;

    //     try{
    //         $modules = $db->loadObjectList();

    //         if(count($modules) > 0){
    //             // Percorre cada modulo a ser atualizado
    //             foreach ($modules as $module) {
    //                 $params = json_decode($module->params);

    //                 // echo '<pre>';
    //                 // var_dump($params); exit;

    //                 // Pula atualizacao de modulo pq ja foi atualizado (gatilho para saber se deve ou nao ser atualizado)
    //                 if(!empty($params->mobile_customization)){
    //                     continue;
    //                 }

    //                 // echo 'johnny';
    //                 // exit;

    //                 // Realiza conversao dos nomes de modais e subforms que tinham prefixo 'model1'
    //                 $params->accessibility = $params->model1_accessibility;
    //                 unset($params->model1_accessibility);
    //                 $params->action_links = $params->model1_action_links;
    //                 unset($params->model1_action_links);
    //                 $params->slim_bar_action_links = $params->model1_slim_bar_action_links;
    //                 unset($params->model1_slim_bar_action_links);
    //                 $params->slim_bar_additional_content = $params->model1_slim_bar_additional_content;
    //                 unset($params->model1_slim_bar_additional_content);
    //                 $params->slim_bar_contact_info = $params->model1_slim_bar_contact_info;
    //                 unset($params->model1_slim_bar_contact_info);
    //                 $params->items_manual = $params->model1_items_manual;
    //                 unset($params->model1_items_manual);
    //                 $params->language = $params->model1_language;
    //                 unset($params->model1_language);
    //                 $params->logo = $params->model1_logo;
    //                 unset($params->model1_logo);
    //                 $params->search = $params->model1_search;
    //                 unset($params->model1_search);
    //                 $params->slim_bar_logo = $params->model1_slim_bar_logo;
    //                 unset($params->model1_slim_bar_logo);
    //                 $params->slim_bar_social_media = $params->model1_slim_bar_social_media;
    //                 unset($params->model1_slim_bar_social_media);
    //                 $params->items_customization = $params->model1_items_customization;
    //                 unset($params->model1_items_customization);
    //                 $params->mobile_customization = $params->model1_mobile_customization;
    //                 unset($params->model1_mobile_customization);
    //                 $params->secondary_bar_customization = $params->model1_secondary_bar_customization;
    //                 unset($params->model1_secondary_bar_customization);
    //                 $params->slim_bar_customization = $params->model1_slim_bar_customization;
    //                 unset($params->model1_slim_bar_customization);
    //                 $params->menu_customization = $params->model1_menu_customization;
    //                 unset($params->model1_menu_customization);
    //                 $params->module_position = $params->model1_module_position;
    //                 unset($params->model1_module_position);

    //                 // Converte para json de volta os parametros para salvar no banco
    //                 $params = json_encode($params);
    
    //                 $query = $db->getQuery(true);

    //                 // Realiza update no banco
    //                 $query->update($db->quoteName('#__modules'))
    //                         ->set($db->quoteName('params') . ' = ' . $db->quote($params))
    //                         ->where("id = '{$module->id}'");

    //                 //echo str_replace('#__', 'ext_', $query); exit;

    //                 // Seta query
    //                 $db->setQuery($query);
    //                 $db->execute();

    //             }

    //         }
    //     }
    //     catch(Exception $e){
	// 		throw new Exception('Erro ao tentar converter os registros da versao antiga para nova');
    //     }
    // }

    // /*
    //     TODO: funcao temporaria criada para migrar campos de uma modal para outra, mantendo compatibilidade de versoes para clientes que atualizarem
    //             * Executado a partir da versao 2.1 criada em Abril de 2020
    //             * Remover apos Abril de 2021
    // */
    // function updateVersion2_1(){
    //     // TODO: Antes de iniciar atualizaco no banco, remove arquivos que deixaram de existir
    //     jimport('joomla.filesystem.file');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/modals/model1_slim_bar_menu_customization.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/modals/model1_main_bar_menu_customization.xml');
    //     JFile::delete(JPATH_ROOT . '/modules/mod_nobossheader/modals/model1_secondary_bar_menu_customization.xml');
        
    //     $db = JFactory::getDBO();
    //     $query = $db->getQuery(true);

    //     $query
    //     ->select("id, params")
    //     ->from('#__modules')
    //     ->where("module = 'mod_nobossheader'");

    //     // Seta query
	// 	$db->setQuery($query);
		
	// 	//echo str_replace('#__', 'ext_', $query); exit;

    //     try{
    //         $modules = $db->loadObjectList();

    //         if(count($modules) > 0){
    //             // Percorre cada modulo a ser atualizado
    //             foreach ($modules as $module) {
    //                 $params = json_decode($module->params);

    //                 // Ja existe dados para nova modal: pula atualizacao de modulo pq ja foi atualizado (gatilho para saber se deve ou nao ser atualizado)
    //                 if((!empty($params->model1_mobile_customization)) || (!empty($params->mobile_customization))){
    //                     continue;
    //                 }

    //                 // TODO: INICIO: dados a migrar da modal de personalizacao da barra principal para nova modal de personalizacao mobile

    //                 // Obtem valores da modal 'model1_items_customization' (barra principal)
    //                 $valuesModalCopiar = json_decode($params->model1_items_customization);

    //                 // Modal onde salvar os dados
    //                 $valuesModalSalvar = new \stdClass();
                        
    //                 // Percorre todos os campos para localizar quais devem ser copiados
    //                 foreach ($valuesModalCopiar as $nameField => $value) {
    //                     // Campo possui 'mobile' no nome
    //                     if(strpos($nameField, 'mobile') !== false){
    //                         // Salva dados em nova modal
    //                         $valuesModalSalvar->{$nameField} = $value;
    //                         // Remove dados da modal antiga
    //                         unset($valuesModalCopiar->{$nameField});
    //                     }
    //                 }

    //                 // Armazena dados de volta da modal a copiar
    //                 $params->model1_items_customization = json_encode($valuesModalCopiar);

    //                 // Armazena dados de volta da modal a salvar
    //                 $params->model1_mobile_customization = json_encode($valuesModalSalvar);
                    
                    
    //                 // TODO: INICIO: dados a migrar da modal de menu da barra principal para a nova modal de personalizaco mobile

    //                 // Obtem valores da modal 'model1_menu_customization' (menu da barra principal)
    //                 $valuesModalCopiar = json_decode($params->model1_menu_customization);
                    
    //                 // Modal onde salvar os dados
    //                 $valuesModalSalvar = json_decode($params->model1_mobile_customization);
                                            
    //                 // Percorre todos os campos para localizar quais devem ser copiados
    //                 foreach ($valuesModalCopiar as $nameField => $value) {
    //                     // Campo possui 'mobile' no nome OU eh um dos campos especificados
    //                     if((strpos($nameField, 'mobile') !== false)
    //                         || ($nameField == 'menu_space')
    //                         || ($nameField == 'menu_icon_size')
    //                         || ($nameField == 'menu_icon_color')
    //                         || ($nameField == 'icons_space')){
    //                             // Salva dados em nova modal
    //                             $valuesModalSalvar->{$nameField} = $value;
    //                             // Remove dados da modal antiga
    //                             unset($valuesModalCopiar->{$nameField});
    //                     }
    //                 }

    //                 // Armazena dados de volta da modal a copiar
    //                 $params->model1_menu_customization = json_encode($valuesModalCopiar);

    //                 // Armazena dados de volta da modal a salvar
    //                 $params->model1_mobile_customization = json_encode($valuesModalSalvar);


    //                 // TODO: INICIO: dados migrados de modal de personalizacao de menu que foi removida para modal ja existente de personalizacao da barra slim

    //                 // Obtem valores da modal 'model1_items_customization' (barra principal)
    //                 $valuesModalCopiar = json_decode($params->model1_slim_bar_menu_customization);
    //                 // Remove modal inteira (foi descontinuada)
    //                 unset($params->model1_slim_bar_menu_customization);

    //                 // Modal onde salvar os dados
    //                 $valuesModalSalvar = json_decode($params->model1_slim_bar_customization);
                        
    //                // Percorre todos os campos para localizar quais devem ser copiados
    //                foreach ($valuesModalCopiar as $nameField => $value) {
    //                     // Campo NAO possui 'mobile' no nome
    //                     if(strpos($nameField, 'mobile') === false){
    //                        // Salva dados em nova modal
    //                        $valuesModalSalvar->{$nameField} = $value;
    //                     }
    //                 }

    //                 // Armazena dados de volta da modal a salvar
    //                 $params->model1_slim_bar_customization = json_encode($valuesModalSalvar);


    //                 // TODO: INICIO: dados migrados de modal de personalizacao de menu que foi removida para modal ja existente de personalizacao da barra principal

    //                 // Obtem valores da modal 'model1_items_customization' (barra principal)
    //                 $valuesModalCopiar = json_decode($params->model1_menu_customization);
    //                 // Remove modal inteira (foi descontinuada)
    //                 unset($params->model1_menu_customization);

    //                 // Modal onde salvar os dados
    //                 $valuesModalSalvar = json_decode($params->model1_items_customization);
                        
    //                // Percorre todos os campos para localizar quais devem ser copiados
    //                foreach ($valuesModalCopiar as $nameField => $value) {
    //                     // Campo NAO possui 'mobile' no nome
    //                     if(strpos($nameField, 'mobile') === false){
    //                        // Salva dados em nova modal
    //                        $valuesModalSalvar->{$nameField} = $value;
    //                     }
    //                 }

    //                 // Armazena dados de volta da modal a salvar
    //                 $params->model1_items_customization = json_encode($valuesModalSalvar);

    //                 // TODO: INICIO: dados migrados de modal de personalizacao de menu que foi removida para modal ja existente de personalizacao da barra principal

    //                 // Obtem valores da modal 'model1_items_customization' (barra principal)
    //                 $valuesModalCopiar = json_decode($params->model1_secondary_bar_menu_customization);
    //                 // Remove modal inteira (foi descontinuada)
    //                 unset($params->model1_secondary_bar_menu_customization);

    //                 // Modal onde salvar os dados
    //                 $valuesModalSalvar = json_decode($params->model1_secondary_bar_customization);
                        
    //                // Percorre todos os campos para localizar quais devem ser copiados
    //                foreach ($valuesModalCopiar as $nameField => $value) {
    //                     // Campo NAO possui 'mobile' no nome
    //                     if(strpos($nameField, 'mobile') === false){
    //                        // Salva dados em nova modal
    //                        $valuesModalSalvar->{$nameField} = $value;
    //                     }
    //                 }

    //                 // Armazena dados de volta da modal a salvar
    //                 $params->model1_secondary_bar_customization = json_encode($valuesModalSalvar);

    //                 // Converte para json de volta os parametros para salvar no banco
    //                 $params = json_encode($params);
    
    //                 $query = $db->getQuery(true);

    //                 // Realiza update no banco
    //                 $query->update($db->quoteName('#__modules'))
    //                         ->set($db->quoteName('params') . ' = ' . $db->quote($params))
    //                         ->where("id = '{$module->id}'");

    //                 //echo str_replace('#__', 'ext_', $query); exit;

    //                 // Seta query
    //                 $db->setQuery($query);
    //                 $db->execute();

    //             }

    //         }
    //     }
    //     catch(Exception $e){
	// 		throw new Exception('Erro ao tentar converter os registros da versao antiga para nova');
    //     }
    // }

    /*
        TODO: funcao temporaria que remove arquivos do formato antigo da extensao antes do Joomla5
                * Criado em Julho de 2024
    */
    // public static function updateJoomla5(){
    //     if(is_file(JPATH_ROOT . '/modules/mod_nobossbanners/mod_nobossbanners.php')){
    //         File::delete(JPATH_ROOT . '/modules/mod_nobossbanners/mod_nobossbanners.php');
    //     }
    //     if(is_file(JPATH_ROOT . '/modules/mod_nobossbanners/helper.php')){
    //         File::delete(JPATH_ROOT . '/modules/mod_nobossbanners/helper.php');
    //     }
    // }
}
